/*
 * Created on Jun 22, 2004
 *
 */
package edu.ncsu.csc.mas.ontology;

import com.hp.hpl.jena.ontology.* ;
import com.hp.hpl.jena.rdf.model.*;

import edu.ncsu.csc.mas.ontology.exceptions.*;


/**
 * A Commitment made by one role to another.
 * 
 * A commitment is a promise that one agent makes to another to bring about a certain 
 * state of affairs. The promise is made by the <code>debtor</code> to the 
 * <code>creditor</code>. THe state of affairs promised is represented by the <code>condition</code>
 * 
 * @author Ashok Mallya
 *
 */
public class Commitment extends OwlpObject{
    
    protected Role debtor ;
    protected Role creditor ;
    protected Condition condition ;
    
    protected StmtIterator sIter ;
   
    /**
     * Return a fresh <code>Commitment</code> object
     */
    public Commitment() {
        
    }
    
    /**
     * Return a fresh <code>Commitment</code> object given the JENA object representing it.
     * 
     * @param res the JENA object representing the commitment
     * @throws MalformedConditionException
     * @throws JenaConversionException
     */
    public Commitment(OntResource res) 
    throws MalformedConditionException
    {
        for(sIter = res.listProperties(); sIter.hasNext();) {
            Statement st = sIter.nextStatement() ;
            Property prop = st.getPredicate();
            RDFNode obj = st.getObject() ;
            OntResource objRes ;
            if(obj.canAs(OntResource.class)) {
                objRes = (OntResource)obj.as(OntResource.class) ;
            } else
                continue ;
            if(prop.getLocalName().equals("hasCreditor")) {
                this.creditor = new Role(objRes) ;
            } else if(prop.getLocalName().equals("hasDebtor")) {
                this.debtor = new Role(objRes) ;
            } else if(prop.getLocalName().equals("hasCondition")) {
                this.condition = new Condition(objRes) ;
            }
        }
    }
    
    public String toString() {
        return new String("C(" + debtor.toString() + "," + creditor.toString() 
                	+ "," + condition.toString() + ") ") ;
        
        
        
    }
    
    public String toJessString(){
    	String retString = "";
    	retString+= ("(bind ?c-var(new Commitment \"" + this.debtor.toString()+ "\"   \"" + this.creditor.toString()+"\"   \"" + this.condition.toString() + "\" ?t))");
    	retString+=("\n\t (definstance com ?c-var static)");
    	return retString;
       
    }

    /**
     * Return the condition (promise) of this commitment
     * 
     * @return Returns the condition.
     */
    public Condition getCondition() {
        return condition;
    }
    /**
     * Set the condition(promise) of this commitment
     * 
     * @param condition The condition to set.
     */
    public void setCondition(Condition condition) {
        this.condition = condition;
    }
    /**
     * Return the creditor of this commitment
     * 
     * @return Returns the creditor.
     */
    public Role getCreditor() {
        return creditor;
    }
    /**
     * Set the creditor of this commitment
     * 
     * @param creditor The creditor to set.
     */
    public void setCreditor(Role creditor) {
        this.creditor = creditor;
    }
    /**
     * Return the debtor of this commitment
     * 
     * @return Returns the debtor.
     */
    public Role getDebtor() {
        return debtor;
    }
    /**
     * Set the debtor of this commitment
     * 
     * @param debtor The debtor to set.
     */
    public void setDebtor(Role debtor) {
        this.debtor = debtor;
    }
}
