/*
 * Created on Jun 22, 2004
 *
 */
package edu.ncsu.csc.mas.ontology;

import com.hp.hpl.jena.ontology.* ;
import com.hp.hpl.jena.rdf.model.*;
import edu.ncsu.csc.mas.ontology.exceptions.*;
import edu.ncsu.csc.mas.utils.Constants;

import java.util.Vector ;


/**
 * @author aumallya
 *
 */
public class Proposition extends OwlpObject{
    
    private String content ;
    private Vector slots ;
    private StmtIterator sIter ;
    
    public Proposition() {
        content = new String("") ;
        slots = new Vector() ;
    }
    
    public Proposition(OntResource node) {
        this.content= node.getLocalName() ;
        this.slots = new Vector() ;
         //list all statements in which this proposition is the subject
        for(sIter=node.listProperties(); sIter.hasNext(); ) {
            //get the next statement in which this prop is the subject
            Statement propSt = sIter.nextStatement();
            //get the object of the statement
            RDFNode objRDFNode = propSt.getObject() ;
            OntResource propObj ;
            //try and convert this object into an OntResource
            if(objRDFNode.canAs(OntResource.class)) {
                propObj = (OntResource)objRDFNode.as(OntResource.class) ;
            } else 
                continue ;
            
           if(propSt.getPredicate().getLocalName().equals(Constants.PROP_SLOT_PROPERTY_NAME)) {
                //one of the slots, i.e., parameters of this proposition
                if(propObj.getRDFType().getLocalName().equals("BooleanSlot"))
                	this.slots.add(new BooleanSlot(propObj)) ;
                else if(propObj.getRDFType().getLocalName().equals("FloatSlot"))
                	this.slots.add(new FloatSlot(propObj)) ;
                else if(propObj.getRDFType().getLocalName().equals("IntegerSlot"))
                	this.slots.add(new IntegerSlot(propObj)) ;
                else if(propObj.getRDFType().getLocalName().equals("StringSlot"))
                	this.slots.add(new StringSlot(propObj)) ;
            } 
                    
        }
    }
    
    public Proposition(Proposition p){
        this.content = new String(p.getContent()) ;
    }
    
    public String toString() {
        String retString = new String("(" + this.content + ")") ;
    /*    if(this.slots!= null) {
            for(int i=0; i<this.slots.size(); i++) {
               	String slot = ((Slot)this.slots.elementAt(i)).toString();
                retString +=  (" ("+ slot + " ?" + slot + "-var)" ) ;
            }

        }

        retString += ") ";*/
        return retString   ;
    }
    
    /**
     * @author pchakra 
     *
     */
    public String toJessString(int num) {
    	
    	String retString="";
    	
    	if(!(this.content.equals("startProp")))    		
    		{ String msgProp=this.content.replaceFirst("Prop","MsgProp");
        	 retString = new String("(" + msgProp + " ") ;}
    	else retString = new String("(" +this.content + "");
       
    	if(this.slots!= null) {
            for(int i=0; i<this.slots.size(); i++) {
               	String slot = ((Slot)this.slots.elementAt(i)).toJessString();
                retString +=  (" ("+ slot + " ?" + slot + "-var" +num +")" ) ;
            }
            if(!(this.content.equals("startProp")))
            	retString+="(transID ?t" +num + ")";
        }

        retString += ") ";
        return retString   ;
    }
    public String toJessStringForCondition() {
        String retString = new String(this.content ) ;
        
                    return retString   ;
    }
    /**
     * @return Returns the content.
     */
    public String getContent() {
        return content;
    }
    /**
     * @param content The content to set.
     */
    public void setContent(String content) {
        this.content = content;
    }
    
    public void addSlot(Slot s) {
        if(this.slots==null)
            this.slots = new Vector() ;
        this.slots.add(s);
    }
    
    public Vector getSlots(){
    	
    	return this.slots;
    }
    
    public boolean equals(Proposition p) {
        System.out.println("Comparing " + p.toString() + " with " + this.toString()) ;
        if(this.toString().equals(p.toString()))
            return true ;
        return false ;
    }
}
