/*
 * Created on Oct 20, 2004
 *
 */
package edu.ncsu.csc.mas.ontology;

import java.util.Vector;

import com.hp.hpl.jena.ontology.ObjectProperty;
import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.util.iterator.NiceIterator;

import edu.ncsu.csc.mas.ontology.exceptions.JenaConversionException;
import edu.ncsu.csc.mas.ontology.exceptions.MalformedConditionException;
import edu.ncsu.csc.mas.utils.Constants;
import edu.ncsu.csc.mas.utils.OwlpGenerator;

/**
 * @author aumallya 
 *
 */
public class SwrlIndividualPropertyAtom extends SwrlAtom {
    
    private OntResource ontArgument1 ;
    private OntResource ontArgument2 ;
    
    private OwlpObject owlpArgument1 ;
    private OwlpObject owlpArgument2 ;
    
    private PropertyPredicate owlpPropertyPredicate ;
    
    private ObjectProperty propertyPredicate ; 
    private OwlpGenerator factory;
    
    private Vector slots = new Vector();
   
    
    public SwrlIndividualPropertyAtom(OwlpObject arg1, OwlpObject arg2, PropertyPredicate prop) {
        this.owlpArgument1 = arg1 ;
        this.owlpArgument2 = arg2 ;
        this.owlpPropertyPredicate = prop ;
        
    }
    
    /**
     * Create a SwrlIndividualPropertyAtom given an OntResource
     * i.e., extract the two argument and the propertyPredicate
     *  
     * @param ontAtom
     */
    public SwrlIndividualPropertyAtom(OntResource ontAtom) {
        //We currently handle only IndividualPropertyAtom
    	System.out.println("calling constructor");
        if(ontAtom.getRDFType().getLocalName().equals("IndividualPropertyAtom") == false)
        {System.out.println("returning since not ind prop atom");
            return ;}
        factory = OwlpGenerator.getOwlpGenerator() ;
        //retrieve argument1
        RDFNode rdfArg ;
        rdfArg = ontAtom.getPropertyValue(factory.getBaseModel().getProperty(Constants.SWRLLocation + Constants.hash+ "propertyPredicate")) ;
        if(rdfArg.canAs(ObjectProperty.class)) {
            propertyPredicate = (ObjectProperty)rdfArg.as(ObjectProperty.class) ;
            this.owlpPropertyPredicate = new PropertyPredicate(propertyPredicate) ;
            //System.out.println(propertyPredicate.getRDFType().getLocalName()) ;
            //if(Constants.DEBUG==true) System.out.print("  " + propertyPredicate.getLocalName() + "( ") ;
        }
        rdfArg = ontAtom.getPropertyValue(factory.getBaseModel().getProperty(Constants.SWRLLocation + Constants.hash + "argument1")) ;
        if(rdfArg.canAs(OntResource.class)) {
            ontArgument1 = (OntResource)rdfArg.as(OntResource.class) ;
            if(ontArgument1.getRDFType().getLocalName().equals("KnowledgeBase")) {
                this.owlpArgument1 = new KnowledgeBase(ontArgument1) ;
            } else if(ontArgument1.getRDFType().getLocalName().equals("Commitment")) {
                Statement stmt = ontArgument1.getProperty(factory.getBaseModel().getProperty(Constants.OntologyFileLocation +  "#hasPrecondition")) ;
                if(stmt != null) {
                    try{
                        this.owlpArgument1 = new ConditionalCommitment(ontArgument1) ;
                    } catch (JenaConversionException jce) {System.err.println(jce) ;}
                    catch (MalformedConditionException mce) {System.err.println(mce) ;}
                } else {
                    try{
                        this.owlpArgument1 = new Commitment(ontArgument1) ;
                    } catch(MalformedConditionException mce) {System.err.println(mce) ;}
                }
            } else if(ontArgument1.getRDFType().getLocalName().equals("Proposition")) {
                this.owlpArgument1 = new Proposition(ontArgument1) ;
            } else if(ontArgument1.getRDFType().getLocalName().equals("Role")) {
                this.owlpArgument1 = new Role(ontArgument1) ;
            } else if(ontArgument1.getRDFType().getLocalName().equals("Message")) {
                try{
                    this.owlpArgument1 = new Message(ontArgument1) ;
                } catch(MalformedConditionException mce) {System.err.println(mce);}
            }
            //System.out.println(ontArgument1.getRDFType().getLocalName()) ;
            //if(Constants.DEBUG==true) System.out.print(ontArgument1.getLocalName() + ", ") ;
        }
        rdfArg = ontAtom.getPropertyValue(factory.getBaseModel().getProperty("http://www.w3.org/2003/11/swrl#argument2")) ;
        if(rdfArg.canAs(OntResource.class)) {
            ontArgument2 = (OntResource)rdfArg.as(OntResource.class) ;
            //System.out.println(ontArgument2.getRDFType().getLocalName());
            if(ontArgument2.getRDFType().getLocalName().equals("KnowledgeBase")) {
                this.owlpArgument2 = new KnowledgeBase(ontArgument2) ;
            } else if(ontArgument2.getRDFType().getLocalName().equals("Commitment")) {
                Statement stmt = ontArgument2.getProperty(factory.getBaseModel().getProperty(Constants.OntologyFileLocation +  "#hasPrecondition")) ;
                if(stmt != null) {
                    try{
                        this.owlpArgument2 = new ConditionalCommitment(ontArgument2) ;
                    } catch (JenaConversionException jce) {System.err.println(jce) ;}
                    catch (MalformedConditionException mce) {System.err.println(mce) ;}
                } else {
                    try{
                        this.owlpArgument2 = new Commitment(ontArgument2) ;
                    } catch(MalformedConditionException mce) {System.err.println(mce) ;}
                }
            } else if(ontArgument2.getRDFType().getLocalName().equals("Proposition")) {
                this.owlpArgument2 = new Proposition(ontArgument2) ;
            } else if(ontArgument2.getRDFType().getLocalName().equals("Role")) {
                this.owlpArgument2 = new Role(ontArgument2) ;
            } else if(ontArgument2.getRDFType().getLocalName().equals("Message")) {
                try{
                    this.owlpArgument2 = new Message(ontArgument2) ;
                } catch(MalformedConditionException mce) {System.err.println(mce);}
            }
            //if(Constants.DEBUG==true) System.out.println(ontArgument2.getLocalName() + " )") ;
        }
        
    }
    
    public String toString() {
        String retString = "";
        retString += this.owlpPropertyPredicate.getName() + "("
        			 + this.owlpArgument1.toString() + ", "
        			 + this.owlpArgument2.toString() + ") ";
                
                
        return retString ;
    }
    
    /**
     * @author pchakra 
     *
     */
    public String toJessRuleString(int num){
    	String retString = "";
    	   	
    	
    	if(this.owlpPropertyPredicate.getName().equals("contains"))
    	{
    		  	
    		Proposition p = new Proposition(this.getOntArgument2());
    		this.slots.addAll(p.getSlots());
    		retString+=p.toJessString(num);
    		
    	}
    	else if(this.owlpPropertyPredicate.getName().equals("createCommitment"))
    	{
    		
    	 	Statement stmt = this.getOntArgument2().getProperty(factory.getBaseModel().getProperty(Constants.OntologyFileLocation +  "#hasPrecondition")) ;
            if(stmt != null) {
               
            	try {
					ConditionalCommitment cc = new ConditionalCommitment(this.getOntArgument2());
								    retString+=cc.toJessString();
				} catch (MalformedConditionException e) {
					System.err.println(e);
				} catch (JenaConversionException e) {
					
					System.err.println(e);
					
				}
            
                   
            } else {
                
            		try {
						Commitment c = new Commitment(this.getOntArgument2());
						retString+=c.toJessString();
					} catch (MalformedConditionException e) {
						System.err.println(e);
					
					}
               
            }
       
      	}
    	else if(this.owlpPropertyPredicate.getName().equals("sendMessage"))
    		
    	{
    		System.out.println("Entering send msg");
    		try {
				Message m = new Message(this.getOntArgument2());
				this.slots.addAll(m.getParameters());
				retString+=m.toSendMsgJessString(num);
			} catch (MalformedConditionException e) {
				
				System.err.println(e);
			}
    	}
    	else if(this.owlpPropertyPredicate.getName().equals("receiveMessage"))
    	{
    		System.out.println("Entering recv msg");
    		try {
				Message m = new Message(this.getOntArgument2());
				this.slots.addAll(m.getParameters());
				retString+=m.toRecvMsgJessString(num);
				
				
			} catch (MalformedConditionException e) {
				
				System.err.println(e);
			}
    
    	}
    	else if(this.owlpPropertyPredicate.getName().equals("assertToKB"))
    	{
    		retString="";
    	}
    		
    		
    		else 
    		{
    		
    		retString+=this.toString();
    		}
    	
    	return retString;
    	
    	
    }
    

    /**
     * @return Returns the ontArgument1.
     */
    public OntResource getOntArgument1() {
        return ontArgument1;
    }
    /**
     * @param ontArgument1 The ontArgument1 to set.
     */
    public void setOntArgument1(OntResource argument1) {
        this.ontArgument1 = argument1;
    }
    /**
     * @return Returns the ontArgument2.
     */
    public OntResource getOntArgument2() {
    	if(this.ontArgument2==null)
    	{
    		
    		String owlpArgClass = this.owlpArgument2.getOntClassFromOwlpObject();
    		OwlpGenerator factory = OwlpGenerator.getOwlpGenerator();
        	OwlpObject owlpArg2 = this.getOwlpArgument2();               
        	OntClass clas = (OntClass)factory.baseModel.getOntClass(factory.getOWLPURI()+ owlpArgClass) ;
            
            boolean found =false;
            
            OntResource inst = null ;
            NiceIterator nIter1;
    		for(nIter1 = (NiceIterator)clas.listInstances() ; nIter1.hasNext();) {
    		    inst = (OntResource)nIter1.next()  ;
    		
    		    
    		   if(owlpArgClass.equals("Proposition"))
    		    {
    		       if(inst.getLocalName().equals(((Proposition)owlpArg2).getContent()))
    		       {	found=true;
    		       
    		       break;}
    		       
    		       
    		           
    		       
    		    }  	    
    		       
    		      if(owlpArgClass.equals("Role"))
    		    {
    		    	if(inst.getLocalName().equals(((Role)owlpArg2).getName()))
    		    	{	found=true;
     		       
     		       break;}
    		    		
    		    }
    		    
    		    if(owlpArgClass.equals("KnowledgeBase"))
    		    {
    		    	if(inst.getLocalName().equals(((KnowledgeBase)owlpArg2).getName()))
    		    	{	found=true;
     		       
     		       break;}
    		    		
    		    }
    		   
    		    if(owlpArgClass.equals("Message"))
    		    {
    		    	if(inst.getLocalName().equals(((Message)owlpArg2).getType()))
    		    	{	found=true;
     		       
     		       break;}
    		    		
    		    		
    		    }	   
    		}  
    		    if(found==true)
    		    {   this.setOntArgument2(inst);
    		    System.out.println("setting null ont arg to: " + inst);
    		    }   		       	   
    		    		
    		
    		
    		
    	}
        return ontArgument2;
    }
    /**
     * @param ontArgument2 The ontArgument2 to set.
     */
    public void setOntArgument2(OntResource argument2) {
        this.ontArgument2 = argument2;
    }
    /**
     * @return Returns the propertyPredicate.
     */
    public PropertyPredicate getPropertyPredicate() {
        return owlpPropertyPredicate;
    }
    /**
     * @param propertyPredicate The propertyPredicate to set.
     */
    public void setPropertyPredicate(PropertyPredicate propertyPredicate) {
        this.owlpPropertyPredicate = propertyPredicate;
    }
    /**
     * @return Returns the owlpArgument1.
     */
    public OwlpObject getOwlpArgument1() {
        return owlpArgument1;
    }
    /**
     * @param owlpArgument1 The owlpArgument1 to set.
     */
    public void setOwlpArgument1(OwlpObject owlpArgument1) {
        this.owlpArgument1 = owlpArgument1;
    }
    /**
     * @return Returns the owlpArgument2.
     */
    public OwlpObject getOwlpArgument2() {
        return owlpArgument2;
    }
    /**
     * @param owlpArgument2 The owlpArgument2 to set.
     */
    public void setOwlpArgument2(OwlpObject owlpArgument2) {
        this.owlpArgument2 = owlpArgument2;
    }
    /**
     * @return Returns the owlpPropertyPredicate.
     */
    public PropertyPredicate getOwlpPropertyPredicate() {
        return owlpPropertyPredicate;
    }
    /**
     * @param owlpPropertyPredicate The owlpPropertyPredicate to set.
     */
    public void setOwlpPropertyPredicate(PropertyPredicate owlpPropertyPredicate) {
        this.owlpPropertyPredicate = owlpPropertyPredicate;
    }
    /**
     * @param propertyPredicate The propertyPredicate to set.
     */
    public void setPropertyPredicate(ObjectProperty propertyPredicate) {
        this.propertyPredicate = propertyPredicate;
    }
    
    public Vector getSlots()
    {
    	return this.slots;
    }
    
   
    
    public boolean equals(SwrlIndividualPropertyAtom sAtom) {
        if(this.toString().equals(sAtom.toString()))
            return true;
        else return false;
    }
}
