/*
 * Created on Jun 24, 2004
 *
 */
package edu.ncsu.csc.mas.ontology.exceptions;


/**
 * Signals an ill-formed condition in an OWL-P instance document.
 * 
 * <a href="http://bombadil.csc.ncsu.edu/owl/v3_6/Protocol.owl">OWL-P</a> specifies a condition
 * as having conjuncts, disjuncts, negation, or a proposition. However, OWL-P does not specify
 * the following: <i>A condition can be one of a conjunction, a disjunction, a negation, or a 
 * proposition</i>. This exception is thrown when the above rule is violated by any OWL-P
 * instance document. 
 * 
 * @author Ashok Mallya
 *
 */
public class MalformedConditionException extends OwlpException {
    
    String conditionName ;
    
    /**
     * Return a fresh <code>MalformedException</code> object
     *
     * @return a fresh <code>MalformedException</code> object
     */
    public MalformedConditionException() {
        super() ;
        this.conditionName = new String("") ;
    }
    
    /**
     * Return a fresh <code>MalformedException</code> object given the name of ill-formed
     * condition.
     *
     * @return a fresh <code>MalformedException</code> object
     */
    public MalformedConditionException(String s) {
        this.conditionName = new String(s) ;
    }
    
    public String toString() {
        return new String( "Ill-formed Condition "+ conditionName) ;
    }

}
