/*
 * Created on Jun 22, 2004
 *
 */
package edu.ncsu.csc.mas.utils;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.ontology.* ;
import com.hp.hpl.jena.util.iterator.*;
import java.util.* ;

import edu.ncsu.csc.mas.ontology.exceptions.*;
import edu.ncsu.csc.mas.ontology.Protocol;


/**
 * @author aumallya
 *
 */
public class OwlpGenerator {

    private String OWLPFileLocation ;
	private String instanceFileLocation ;
	private String swrlImpURI ;
	private String instanceURI ;
	private String OWLPURI ;
	public OntModel baseModel ;
	public OntDocumentManager baseDocManager ;
	private NiceIterator nIter1, nIter2 ;
	private Iterator iter ;
	private StmtIterator sIter ;

	private static OwlpGenerator theOne = null;
	
	private OwlpGenerator () {
	}
	
	private OwlpGenerator(String ontFileLoc, String instFileLoc) {
	    if(Constants.DEBUG==true) System.out.println("Constructing OwlpGenerator") ;
	    this.setFileLocations(ontFileLoc, instFileLoc) ;
	}
	
	private void setFileLocations(String OntologyFileLocation, String InstanceFileLocation) {
	    if(Constants.DEBUG==true) System.out.println("Setting file names and locations") ;
	    this.OWLPFileLocation = OntologyFileLocation ;
	    if(Constants.DEBUG==true) System.out.println("OWLPFileLocation = " + OntologyFileLocation) ;
	    this.instanceFileLocation = InstanceFileLocation ;
	    if(Constants.DEBUG==true) System.out.println("InstanceFileLocation = " + InstanceFileLocation) ;
		this.instanceURI = new String(instanceFileLocation + Constants.hash) ;
		this.OWLPURI = new String(OWLPFileLocation + Constants.hash) ;
		this.swrlImpURI = Constants.SWRLLocation +  "#Imp" ;
		this.baseModel = ModelFactory.createOntologyModel() ;
		this.baseDocManager = this.baseModel.getDocumentManager() ;
		this.baseDocManager.setProcessImports( true) ;
	    if(Constants.DEBUG==true) System.out.println("Creating JENA Model") ;
		this.baseModel.read(this.instanceURI) ;
	}
	
    
    /**
     * @return Returns the instanceURI.
     */
    public String getInstanceURI() {
        return instanceURI;
    }
    /**
     * @param instanceURI The instanceURI to set.
     */
    public void setInstanceURI(String baseNamespaceURI) {
        this.instanceURI = baseNamespaceURI;
    }
    /**
     * @return Returns the baseDocManager.
     */
    public OntDocumentManager getBaseDocManager() {
        return baseDocManager;
    }
    /**
     * @return Returns the baseModel.
     */
    public OntModel getBaseModel() {
        return baseModel;
    }
    
   
    /**
     * @return Returns the oWLPURI.
     */
    public String getOWLPURI() {
        return OWLPURI;
    }
    /**
     * @param owlpuri The oWLPURI to set.
     */
    public void setOWLPURI(String owlpuri) {
        OWLPURI = owlpuri;
    }
    
    public Protocol generateProtocol() 
    throws MalformedConditionException
    {
        OntClass clas = (OntClass)baseModel.getOntClass(getOWLPURI()+ Constants.PROTOCOL_CLASS_NAME) ;
        OntResource inst = null ;
		for(nIter1 = (NiceIterator)clas.listInstances() ; nIter1.hasNext();) {
		    inst = (OntResource)nIter1.next()  ;
		    System.out.println("OntResources in the protocol: " + inst);
		}
		return new Protocol(inst) ;
    }
    
    public static OwlpGenerator getOwlpGenerator(){
        if(theOne==null) {
            if(Constants.DEBUG==true) System.out.println("Creating a new OwlpGenerator") ;
            theOne = new OwlpGenerator(Constants.OntologyFileLocation, Constants.InstanceFileLocation) ;
        }
        return theOne ;
    }

    public static OwlpGenerator getOwlpGenerator(String instURI){
        if(theOne==null) {
            if(Constants.DEBUG==true) System.out.println("Creating a new OwlpGenerator") ;
            theOne = new OwlpGenerator(Constants.OntologyFileLocation, instURI) ;
        }
        return theOne ;
    }
    
    
    /**
     * @return Returns the swrlImpURI.
     */
    public String getSwrlImpURI() {
        return swrlImpURI;
    }
    /**
     * @param swrlImpURI The swrlImpURI to set.
     */
    public void setSwrlImpURI(String swrlImpURI) {
        this.swrlImpURI = swrlImpURI;
    }
}
