/*
 * Created on Jun 24, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */

/**
 * @author Ashok Mallya
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */

package edu.ncsu.csc.mas.utils ;

import java.util.Iterator;
import java.util.Vector;

import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.iterator.NiceIterator;

import edu.ncsu.csc.mas.ontology.MessageInQueue;
import edu.ncsu.csc.mas.ontology.Message;
import edu.ncsu.csc.mas.ontology.OwlpObject;
import edu.ncsu.csc.mas.ontology.Policy;
import edu.ncsu.csc.mas.ontology.PropertyPredicate;
import edu.ncsu.csc.mas.ontology.Protocol;
import edu.ncsu.csc.mas.ontology.Role;
import edu.ncsu.csc.mas.ontology.Rule;
import edu.ncsu.csc.mas.ontology.SwrlIndividualPropertyAtom;
import edu.ncsu.csc.mas.ontology.exceptions.*;

/**
 * SkeletonGenerator implements the algorithm to extract rules that pertain to
 * a role in the protocol such that the rules are self contained.
 * 
 * 1. A role knows only what it can see because of messages that is can percieve
 * 2. A role can percieve only those messages that it either sends or receives 
 * 
 * @author aumallya
 *
 */

public class SkeletonGenerator {
	
	/**
	 * Gather all rules that cause a message to be sent to or by this role 
	 * 
	 * @param prot Protocol in which this role is defined
	 * @param role the role for which skeleton is to be generated
	 * @return Vector of messages for the skeleton of Role role
	 */
	public Vector getAffectingRules(Protocol prot, Role role) 
	{
	    System.out.println("------------------------------\nGathering rules for role:" + role.getName() + "\n------------------------------");
	    
	    
	    if(prot==null)
	        return null;

	    Vector rules = prot.getRules() ;

	    if(rules==null)
	        return null ;
	    
	    if(rules.size()<1)
	        return null;
	    
		Vector rulesForRole = new Vector() ;
		
		for(int i=0; i<rules.size(); i++)
		{
		    /**
		     * For each rule in the protocol, if role appears as a sender
		     * or receiver of a message on the RHS (i.e., in the head of 
		     * the rule), copy that rule into the return Vector after 
		     * appropriately changing send/receive 
		     **/
		    //get the next rule
		    Rule currentRule = (Rule)rules.get(i) ;
		  //  System.out.println("Rule from rules: "+ currentRule);
		    //Create a rule for the skeleton
		    Rule returnRule = new Rule();
            //The body will be unchanged, for now.
           returnRule.setBody(currentRule.getBody());
		   // Vector originalBody = new Vector();
		  // originalBody = currentRule.getBody();
	    	returnRule.setName(currentRule.getName() + "_for_" + role.getName());
		    //get the head of the rule
		    Vector ruleHead = currentRule.getHead() ;
		    if(ruleHead == null)
		        continue ;
		    if(ruleHead.size() < 1)
		        continue ;
		    
	        //Is role either a sender or receiver of any sendMessage atom in the head?
	        //if it is not, we do not need to copy this rule
		    boolean roleInvolved =false;
		   
		    /**
		     * For each atom in the rule head, if it is a sendMessage atom,
		     * then check if role is the sender or the receiver in that atom
		     */
		    for(int j=0; j < ruleHead.size(); j++) {
		        
		        SwrlIndividualPropertyAtom swrlAtom ;
		        swrlAtom = (SwrlIndividualPropertyAtom)ruleHead.get(j) ;
		            
		        
		        if(swrlAtom.getPropertyPredicate().getName().equals("sendMessage")){
		            /* We have found a sendMessage atom. Now we have to find the sender and receiver of this atom. */
		 //           
		        	if(swrlAtom.getOwlpArgument2() instanceof Message) {
		                Message mS = (Message)swrlAtom.getOwlpArgument2() ;
		                
		        		if(mS.getSender().getName().equals(role.getName()))
		        		{
		        			roleInvolved = true;
		        			
		        			
		        			//System.out.println(role.getName()+ " is a sender in " + currentRule + " hence sendMsg" );
		        			     		        			 
		        			 
		        		}
		        		else if(mS.getReceiver().getName().equals(role.getName()))
		        		{
		        			roleInvolved = true;
		        		
		        			//System.out.println(role.getName()+ " is a receiver in " + currentRule + " hence recvMsg" );
		        			Message msg = new Message(mS.getType(), null, null, mS.getParameters());
		        			swrlAtom = new SwrlIndividualPropertyAtom(new Role(mS.getReceiver()), 
		        					msg, 
         							 new PropertyPredicate("receiveMessage"));                  
		                 //  System.out.println("swrlAtom in recv msg: " + swrlAtom.toString());
		                   
		                   MessageInQueue mq= new MessageInQueue(msg);
		                    returnRule.addToBody(mq);
		                //    System.out.println("Body in skel generator: " + returnRule.getBody().toString());
		                    
		                    		
		                   
		        		}
		                   
		                    	        	
		        	            returnRule.addtoHead(swrlAtom) ;
		            } 
		        }
		         else 
		            returnRule.addtoHead(swrlAtom) ;
		    }
		    //we have now checked all atoms in the head, add the rule
	        if(roleInvolved == true) {
	        	Policy policy= new Policy(returnRule);
	        	System.out.println("Policy for this rule: "+ policy);
	        	returnRule.addToBody(policy);
	            rulesForRole.add(returnRule) ;
	            System.out.println("Return Rule: "+returnRule.toString()) ;
	           
	        }
	        
		}
		   
	return rulesForRole;
		    
	}
	
	/**
	 * Generate the set of rules for a role, taking care of send/receive and also
	 * local asserts of proposiions used in rule bodies
	 * 
	 * @param prot
	 * @param role
	 * @return
	 */
	public Vector generateSkeleton(Protocol prot, Role role) {
	    if(prot==null || role==null)
	        return null;
	
	    /*
	     * First, gather all the pertinent rules; i.e, those rules which cause this role
	     * to send or receives a message
	     */
	    Vector pertinentRules = this.getAffectingRules(prot, role) ;
	    Vector ruleBody, ruleHead , returnRuleSet;
	    String ruleName ;

	    if(pertinentRules==null)
	        return  null;
	    
	    returnRuleSet = new Vector() ;
	    role.setSkelRules(pertinentRules) ;
	    
	    for(int i=0; i<pertinentRules.size(); i++) {
	        /*
	         * ruleBody will change after each iteration of the for(j) loop
	         * because of replacements
	         */
	        ruleBody = ((Rule)pertinentRules.elementAt(i)).getBody() ;
	        ruleHead = ((Rule)pertinentRules.elementAt(i)).getHead() ;
	        ruleName = ((Rule)pertinentRules.elementAt(i)).getName() ;
	        	     
	      
	        if(ruleBody==null)
	            continue ;
	        
	        for(int j=0; j<((Rule)pertinentRules.elementAt(i)).getBody().size(); j++) {
	            
	        	if((((Rule)pertinentRules.elementAt(i)).getBody().elementAt(j)) instanceof SwrlIndividualPropertyAtom)
	        	{ SwrlIndividualPropertyAtom prop = (SwrlIndividualPropertyAtom)((Rule)pertinentRules.elementAt(i)).getBody().elementAt(j) ;
	            
	            if(prop.getOwlpPropertyPredicate().getName().equals("contains")) {
	                Vector replacements = findReplacement(prop.getOwlpArgument2(), role, prot) ;
	                if(replacements==null)
	                    continue ;
	                /* replacemtns.size() will be zero when no replacements need to 
	                 * be done, i.e., the proposition is asserted by a local rule
	                 */
	                ruleBody.remove(j);
	               
	                /*
	                 * replace prop with the replacements, wrapping them as swrl atoms
	                 * findReplacement() returns an array of OwlpObjects, which are 
	                 * propositions. These are proposition found in the head of rules, 
	                 * which means they are part of a "contains" predicate (specifically,
	                 * argument2 of the "contains" predicate). Hence, we create a new
	                 * SwrlIndividualPropertyAtom with arg1=KB, arg2=OwlpObject and 
	                 * propertyPredicate=contains
	                 * 
	                 */
	                for(int k=0; k< replacements.size(); k++) {
	                	
	                	System.out.println("Replacements at "+k+": "+((OwlpObject)replacements.elementAt(k)).toString());
	                	SwrlIndividualPropertyAtom newSwrlAtom = new SwrlIndividualPropertyAtom(prot.getKB(), (OwlpObject)replacements.elementAt(k), prop.getOwlpPropertyPredicate()) ;
	                    
	                	
	                	
	                	ruleBody.add(0,newSwrlAtom) ;
	                }
	            }
	            
	        
	        
	        Rule newRule= new Rule(ruleName, ruleBody, ruleHead);
	        returnRuleSet.add(newRule) ;
	        System.out.println("New rule after replacement: "+ newRule.toString());
	        	}
	        
	      
	        } 
	      
           
	    }
	    role.setSkelRules(returnRuleSet) ;
	    return returnRuleSet ;
	    
	}
	
	/**
	 * Find a replacement for the proposition in the skeleton for role within the 
	 * protocol. 
	 * 
	 * The OWL-P specification of a protocol is a centralized view of the 
	 * protocol. To enact a protocol, skeletons, which are one particular role's view 
	 * of the protocol, have to be derived. This is done with the observation that a
	 * role's knowledge of the environment is gained only through the messages that it
	 * sends or receives. If the environment is changed because of messages that a role
	 * neither sent nor received, then, none of the propositions asseted to the KB
	 * (global KB) by that message will not be seen by the role. Hence, if any rule has
	 * that proposition in its body, we have to replace the proposition by the last
	 * proposition in the conversation that this role saw. 
	 * 
	 * @param proposition The proposition for which a erplacement("last seen") 
	 * proposition has to be found
	 * 
	 * @param role The role for which this skeleton is being built
	 * 
	 * @param prot The protocol for which we are building a skeleton
	 * 
	 * @return A vector of OwlpObjects to replace the proposition with
	 */
	public Vector findReplacement(OwlpObject proposition, Role role, Protocol prot) {
	    if(role==null || prot==null)
	        return null;
	    
	    if(role.getSkelRules()==null)
	        return null;
	 
        System.out.println("***\nFinding replacement for " + proposition + " for " + role) ;
   	 
	    /* Create a vector of OwlpObjects to return
	     * The default return value an empty vector */
	    Vector replacementProps = new Vector();
	    
	    /* The proposition is asserted by one of local rules, so no replacement 
	     * is necessary because we invoke the findReplacement() method on every prop
	     * in the body of every local rule */
	    int localIndex = isAssertedIn(proposition, role.getSkelRules()) ; 
	    
	    if(localIndex !=-1) {
	        System.out.println(proposition + " found Locally in rule " + ((Rule)role.getSkelRules().elementAt(localIndex)).getName()) ;
	        replacementProps.add(proposition) ;
	        System.out.println("Returning " + replacementProps) ;
	        return replacementProps;
	    } 
	    System.out.println("Not found locally, searching global...") ;
	    

	    /* The proposition is not asserted by any local rule, so we need to
	     * find a replacement for it in the global (protocol) rules
	     */
	    int protRuleIndex = isAssertedIn(proposition, prot.getRules()) ;
	    if(protRuleIndex !=-1) {
	        Rule assertingGlobalRule = (Rule)(prot.getRules().elementAt(protRuleIndex)) ;
	        System.out.println("Found Globally in rule " + assertingGlobalRule.getName()) ;
	        Vector ruleBody = (Vector) assertingGlobalRule.getBody() ;
	        for(int i=0; i<ruleBody.size(); i++) {
	            /* If the ruleBody has a proposition of the form contains(_KB_,_prop_)
	             * then find a replacement for that prop */
	            if(((SwrlIndividualPropertyAtom)ruleBody.elementAt(i)).getOwlpPropertyPredicate().getName().equals("contains")) 
	                replacementProps.addAll(findReplacement(((SwrlIndividualPropertyAtom)ruleBody.elementAt(i)).getOwlpArgument2(), role, prot)) ;    
	        }
	        System.out.println("Returning " + replacementProps);
	        return replacementProps ;
	    } 
	            
	   /*
	    * If we are here and havent found the proposition asserted in the local
	    * or the global rules, it means the proposition has to be asserted by some
	    * other means external to the rulebase, so just return it. An example is the
	    * start proposition.
	    */ 
	    replacementProps.add(proposition) ;
        System.out.println("Prop not found, Returning" + replacementProps) ;
	    return replacementProps ;
	    
	}

	/**
	 * Check if the given proposition is asserted (i.e., written to KB) by any of the
	 * given rules. 
	 * 
	 * @param proposition The proposition whose assertion has to be checked
	 * @param rules The vetor of rules which is to be checked
	 * @return The index of the rule that aserts the proposition
	 */
	public int isAssertedIn(OwlpObject proposition, Vector rules) {
	    if(rules==null)
	        return -1;
	    
	    /* Check every rule to see if it has an "assertToKB" property with proposition
	     * as its first parameter
	     */
	    for(int i=0; i< rules.size(); i++) {
	        /*
	         * assertToKB is done only in the head of rules, hence get the head 
	         * of this rule 
	         */
	        Vector ruleHead = ((Rule)rules.elementAt(i)).getHead() ;
	        
	        if(ruleHead == null)
	            return -1 ;
	        
	        for(int j=0; j<ruleHead.size(); j++) {

	            SwrlIndividualPropertyAtom tempSwrlAtom = (SwrlIndividualPropertyAtom)ruleHead.elementAt(j);
	            PropertyPredicate tempPred = tempSwrlAtom.getOwlpPropertyPredicate() ;
	            if(tempPred.toString().equals("assertToKB") == false)
	                continue ;
	            OwlpObject tempProp =  tempSwrlAtom.getOwlpArgument1() ;
	        

	            
	            /*
	             * if(property == "assertToKB" and this property pertains to proposition)
	             * then this rule asserts proposition. return the index of this rule
	             */
	            if( tempPred.toString().equals("assertToKB")  &&  tempProp.toString().equals(proposition.toString()) ) {
	                //System.out.println( proposition.toString() + " is asserted in rule" + rules.elementAt(i)) ;
	                return i ;
	            }
	        }
	    }
	    
	    /*
	     * proposition was not asserted.
	     */
	    return -1 ;
	}
}
	


